#pragma once

namespace CRAFTYCLICKS {

using namespace System;
using namespace System::IO;
using namespace System::Xml;
using namespace System::Xml::Serialization;

public ref class DeliveryPoint
{
public:
    String^ organisation_name;
    String^ department_name;
    String^ line_1;
    String^ line_2;
    String^ line_3;
    String^ udprn;
};

public ref class AddressDataFormatted
{
public:
    array<DeliveryPoint^>^ delivery_points;
    array<String^>^ list_strings;
    int     delivery_point_count;
    String^ town;
    String^ postal_county;
    String^ traditional_county;
    String^ postcode;
};

public ref class CraftyClicks {
    public: 
    
    AddressDataFormatted^ response;
    String^ error_msg;
    String^ access_token;

    void SetAccessToken(System::String^ token) {
        this->access_token = token;
    }

    void LookupPostcode(System::String^ postcode) {

        XmlDocument^ xmlDoc = gcnew XmlDocument;
        String^ URI = gcnew String(L"");
        URI = URI->Format("http://pcls1.craftyclicks.co.uk/xml/rapidaddress?postcode={0}&key={1}&response=data_formatted&lines=3", postcode, this->access_token);
        XmlTextReader^ reader;
        
		this->response = gcnew AddressDataFormatted;
		this->response->delivery_point_count = 0;

		try {
			reader = gcnew XmlTextReader(URI);
			reader->WhitespaceHandling = WhitespaceHandling::None;
			reader->MoveToContent();
		}
		//catch(XmlException e) {
		catch (...) {
		   this->error_msg = L"Could not fetch data from lookup server...";
		   return;
		}

		xmlDoc->Load(reader);

		// step trough the XML and unpack it to a nice object struct
		XmlNode^ root = xmlDoc->FirstChild->FirstChild;
		int del_points = 0;

		if ( root->HasChildNodes ) {
			// is this an error? 
			if (L"error_code" == root->LocalName || L"error_msg" == root->LocalName) {
				root = xmlDoc->FirstChild;
				this->error_msg = root[L"error_msg"]->InnerText;
				this->response->delivery_point_count = 0;
			} else {
				this->error_msg = L"";
				// get common into
				this->response->delivery_point_count = System::Int32::Parse(root[L"delivery_point_count"]->InnerText);
				this->response->town                 = root[L"town"]->InnerText;
				this->response->postal_county        = root[L"postal_county"]->InnerText;
				this->response->traditional_county   = root[L"traditional_county"]->InnerText;
				this->response->postcode             = root[L"postcode"]->InnerText;

				// get all delivery points now
				this->response->delivery_points = gcnew array<DeliveryPoint^>(this->response->delivery_point_count);
				this->response->list_strings = gcnew array<String^>(this->response->delivery_point_count);
				for ( int i = 0; i < root->ChildNodes->Count; i++ ) {
					XmlNode^ node = root->ChildNodes[ i ];
					if (L"delivery_point" == node->LocalName) {
						DeliveryPoint^ delPoint = gcnew DeliveryPoint;
						delPoint->organisation_name  = node[L"organisation_name"]->InnerText;
						delPoint->department_name    = node[L"department_name"]->InnerText;
						delPoint->line_1             = node[L"line_1"]->InnerText;
						delPoint->line_2             = node[L"line_2"]->InnerText;
						delPoint->line_3             = node[L"line_3"]->InnerText;
						delPoint->udprn              = node[L"udprn"]->InnerText;
						this->response->delivery_points[del_points] = delPoint;

						String^ tempStr = String::Copy (delPoint->organisation_name);
						if (0 < tempStr->Length && 0 < delPoint->department_name->Length) {
							tempStr = tempStr->Insert( tempStr->Length, L", " );
						}
						tempStr = tempStr->Insert( tempStr->Length, delPoint->department_name );
						if (0 < tempStr->Length && 0 < delPoint->line_1->Length) {
							tempStr = tempStr->Insert( tempStr->Length, L", " );
						}
						tempStr = tempStr->Insert( tempStr->Length, delPoint->line_1 );
						if (0 < tempStr->Length && 0 < delPoint->line_2->Length) {
							tempStr = tempStr->Insert( tempStr->Length, L", " );
						}
						tempStr = tempStr->Insert( tempStr->Length, delPoint->line_2 );
						if (0 < tempStr->Length && 0 < delPoint->line_3->Length) {
							tempStr = tempStr->Insert( tempStr->Length, L", " );
						}
						tempStr = tempStr->Insert( tempStr->Length, delPoint->line_3 );
						if (0 < tempStr->Length && 0 < this->response->town->Length) {
							tempStr = tempStr->Insert( tempStr->Length, L", " );
						}
						tempStr = tempStr->Insert( tempStr->Length, this->response->town );
						this->response->list_strings[del_points] = tempStr;

						del_points++;
					}
				}
			}
		}
    }

};
};
